#include "PreCompile.h"
#include "LiteMAN.h"
#include "SGTShellFunc.h"


#pragma data_seg(".sdata")
  BOOL g_bHasLoaded = FALSE;
#pragma data_seg()

DWORD     g_DDEInst = 0;
HINSTANCE g_hInst = NULL;


HSZ g_hszProgman = NULL, g_hszGroups = NULL, g_hszFolders = NULL, g_hszAppProperties = NULL;

HDDEDATA CALLBACK DdeCallback(UINT wType, UINT wFmt, HCONV hConv, HSZ hszTopic,
                              HSZ hszItem, HDDEDATA hData, DWORD lData1, DWORD lData2);

char* StripCommand(char*& szCommand);

char* MatchCommand(char*& szCommand);

void * __cdecl operator new(unsigned int uSize)
{
  return HeapAlloc(GetProcessHeap(),0,uSize);
}

void __cdecl operator delete(void * pv)
{
  if(pv)
    HeapFree(GetProcessHeap(),0,pv);
}

int __cdecl _purecall(void)
{
  return 0;
}


BOOL LiteMANInstall(HINSTANCE hInstance)
{
  if (TRUE == g_bHasLoaded) // look to see if we have been installed before
  {
    MessageBox(NULL, "DDE Server already installed", "LiteMAN", MB_ICONEXCLAMATION);
    return (FALSE);
  }

  CSGTShellFunc* psh = CSGTShellFunc::Instance();
  if (psh == NULL)
  {
    MessageBox(NULL, "Unable to allocate SGTShellFunc", "LiteMAN", MB_ICONEXCLAMATION);
    return (FALSE);
  }

  if (NULL != FindWindow("Progman", NULL))
  {
    int nRet = MessageBox(NULL, "LiteMAN has detected a PROGMAN instance!\n"
                                "Would you like to continue?\n\n"  
                                "If you are using LiteStep on top of explorer\n"
                                "please press 'No'.", "LiteMAN", MB_YESNO);

    if (IDNO == nRet)
    {
      return (FALSE);
    }
  }


  if (DMLERR_NO_ERROR != DdeInitialize(&g_DDEInst, (PFNCALLBACK)MakeProcInstance((FARPROC)DdeCallback, hInstance),
            APPCLASS_STANDARD | CBF_FAIL_POKES | CBF_FAIL_SELFCONNECTIONS | CBF_SKIP_ALLNOTIFICATIONS
            , 0L))
  {
    MessageBox(NULL, "Unable to initialise DDE", "LiteMAN", MB_ICONEXCLAMATION);
    return(FALSE);
  }


  g_hszProgman        = DdeCreateStringHandle(g_DDEInst, "PROGMAN", 0);
  g_hszGroups         = DdeCreateStringHandle(g_DDEInst, "Groups", 0);
  g_hszFolders        = DdeCreateStringHandle(g_DDEInst, "Folders", 0);
  g_hszAppProperties  = DdeCreateStringHandle(g_DDEInst, "AppProperties", 0);

  if (0L == g_hszProgman || 0L == g_hszGroups || 0L == g_hszFolders || 0L == g_hszAppProperties)
  {
    MessageBox(NULL, "Unable to create DDE string handles", "LiteMAN", MB_ICONEXCLAMATION);
    return(FALSE);
  }

  if (0L == DdeNameService(g_DDEInst, g_hszProgman, 0L, DNS_REGISTER))
  {
    MessageBox(NULL, "Unable to initilise DDE PROGMAN service", "LiteMAN", MB_ICONEXCLAMATION);
    return (FALSE);
  }
  if (0L == DdeNameService(g_DDEInst, g_hszFolders, 0L, DNS_REGISTER))
  {
    MessageBox(NULL, "Unable to initilise DDE Folders service", "LiteMAN", MB_ICONEXCLAMATION);
    return (FALSE);
  }
  g_bHasLoaded = TRUE;
  g_hInst = hInstance; // Successful install, this is the instance of the DDE Server
  return (TRUE);
}


void LiteMANRemove()
{
  if (NULL != g_hszProgman)
  {
    DdeFreeStringHandle(g_DDEInst, g_hszProgman);
  }
  if (NULL != g_hszGroups)
  {
    DdeFreeStringHandle(g_DDEInst, g_hszGroups);
  }
  if (NULL != g_hszFolders)
  {
    DdeFreeStringHandle(g_DDEInst, g_hszFolders);
  }
  if (NULL != g_hszAppProperties)
  {
    DdeFreeStringHandle(g_DDEInst, g_hszAppProperties);
  }
  DdeUninitialize(g_DDEInst);
  CSGTShellFunc::Destroy();
}

int initModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType* wd)
{
	return initModuleEx (ParentWnd, dllInst, wd->lsPath);
}

int initModuleEx(HWND parent, HINSTANCE dll, LPCSTR szPath)
{
  if (FALSE == LiteMANInstall(dll))
  {
    return (1);
  }

  return (0);
}

void quitModule(HINSTANCE dllInst)
{
  LiteMANRemove();
}







char* StripCommand(char*& szCommand)
{
  int nLen = lstrlen(szCommand);
  szCommand[nLen - 2] = '\0';   // Strip off trailing ")]"
  char* szRet = szCommand + 1;

  szCommand = StrChr(szCommand, '(');
  *(szCommand++) = '\0';       // Separate Command/Parameters

  CharUpperBuff(szRet, lstrlen(szRet));
  
  return (szRet);
}

char* MatchCommand(char*& szCommand)
{
  if (StrStr(szCommand, "[CreateGroup(") == szCommand)
  {
    return (StripCommand(szCommand));
  }

  if (StrStr(szCommand, "[DeleteGroup(") == szCommand)
  {
    return (StripCommand(szCommand));
  }

  if (StrStr(szCommand, "[ShowGroup(") == szCommand)
  {
    return (StripCommand(szCommand));
  }

  if (StrStr(szCommand, "[AddItem(") == szCommand)
  {
    return (StripCommand(szCommand));
  }

  if (StrStr(szCommand, "[DeleteItem(") == szCommand)
  {
    return (StripCommand(szCommand));
  }
  
  if (StrStr(szCommand, "[ExploreFolder(") == szCommand)
  {
    return (StripCommand(szCommand));
  }
  
  if (StrStr(szCommand, "[ViewFolder(") == szCommand)
  {
    return (StripCommand(szCommand));
  }
  
  if (StrStr(szCommand, "[FindFolder(") == szCommand)
  {
    return (StripCommand(szCommand));
  }        

  return (NULL);
}


/*
 *  DdeCallback()
 */

HDDEDATA CALLBACK DdeCallback(
UINT wType,
UINT wFmt,
HCONV hConv,
HSZ hszTopic,
HSZ hszItem,
HDDEDATA hData,
DWORD lData1,
DWORD lData2)
{
  static HANDLE           hToken;
  static TOKEN_PRIVILEGES tp;
  static LUID             luid;
  CSGTShellFunc*          psh = CSGTShellFunc::Instance();


  if (wType == XTYP_CONNECT)
  {
    return((HDDEDATA)TRUE);
  }

  if (wType == XTYP_WILDCONNECT)
  {
    HDDEDATA hData;
    HSZPAIR FAR *phszp;
    DWORD cb;

    if ((!hszTopic || hszTopic == g_hszProgman) && (!hszItem || hszItem == g_hszProgman))
    {
      if ((hData = DdeCreateDataHandle(g_DDEInst, NULL,
              2 * sizeof(HSZPAIR), 0L, 0, 0, 0)))
      {
          phszp = (HSZPAIR FAR *)DdeAccessData(hData, &cb);
          phszp[0].hszSvc = g_hszProgman;
          phszp[0].hszTopic = g_hszProgman;
          phszp[1].hszSvc = phszp[1].hszTopic = 0;
          DdeUnaccessData(hData);
          return(hData);
      }
    }
    return(0);
  }
  CHAR szBuf[256];

  if ((hszTopic == g_hszGroups || hszTopic == g_hszAppProperties) && wType == XTYP_EXECUTE)
  {
    DdeGetData(hData, (LPBYTE)szBuf, 256, 0);
    szBuf[256 - 1] = TEXT('\0');
    char* pszBuf = szBuf;
    char* pszCommand = MatchCommand(pszBuf);

    if (NULL != pszCommand)
    {
      if (psh->ParseRequest(pszCommand, pszBuf))
      {
        return ((HDDEDATA)DDE_FACK);
      }
    }

    return (0);
  }

  if (wFmt == CF_TEXT)
  {

    if (wType == XTYP_ADVSTART || wType == XTYP_ADVSTOP)
    {
      return((HDDEDATA)TRUE);
    }

    if (hszTopic == g_hszProgman && hszItem == g_hszGroups)
    {
      if (wType == XTYP_REQUEST || wType == XTYP_ADVREQ)
      {
        
        LPVOID pList = NULL;
        UINT ulLen = 0;

        HDDEDATA hDDERet = (HDDEDATA) 0;

        if (psh->ListGroups(pList, ulLen))
        {
          hDDERet = DdeCreateDataHandle(g_DDEInst, (LPBYTE)pList, ulLen, 0L,
                  g_hszGroups, CF_TEXT, 0);
          HeapFree(GetProcessHeap(), 0, pList);
        }
        return (hDDERet);
      }
    }
  }
  return(0);

  UNREFERENCED_PARAMETER(lData1);
  UNREFERENCED_PARAMETER(lData2);
  UNREFERENCED_PARAMETER(hConv);
}

BOOL WINAPI DllMain(HANDLE hDllHandle, DWORD dwReason, LPVOID lpreserved)
{
  return (TRUE);
}
