#include "PreCompile.h"
#include "SGTShellFunc.h"

// class-global pointer to the singleton class.
CSGTShellFunc* CSGTShellFunc::m_pCSGTShellFunc = NULL;

BOOL IsAdmin()
{
  SC_HANDLE hSC;
  hSC = OpenSCManager(NULL, NULL, GENERIC_READ | GENERIC_WRITE | GENERIC_EXECUTE);
  if (hSC == NULL)
    return FALSE;

  CloseServiceHandle(hSC);

  return TRUE;
}


CSGTShellFunc::CSGTShellFunc()
{
  memset (m_szCurrentGroup, 0, MAX_PATH);
  // Do admin check.
  m_bIsUserAnAdmin = IsAdmin();
}

// Singleton instantiation function.
CSGTShellFunc* CSGTShellFunc::Instance()
{
  if (NULL == m_pCSGTShellFunc)
  {
    // make sure OLE is initialised
    HRESULT hr = CoInitialize(NULL);

    if (S_FALSE == hr)
    {
      // already initialised by us, make matching CoUninitialize() call
      // to clean up.
      CoUninitialize();
    }
    else
    if (S_OK != hr)
    {
      return (NULL);
    }
    m_pCSGTShellFunc = new CSGTShellFunc();
  }

  return (m_pCSGTShellFunc);
}

// clean-up function, must be called on a global exit level.
void CSGTShellFunc::Destroy()
{
  if (NULL != m_pCSGTShellFunc)
  {
    CoUninitialize();
    delete (m_pCSGTShellFunc);
  }

  m_pCSGTShellFunc = NULL;
}


CSGTShellFunc::~CSGTShellFunc()
{
}


// Handle incoming DDE requests and pass to relevant functions
BOOL CSGTShellFunc::ParseRequest(char* szRequest, char* pszParameters)
{
  // 10 is the Maximum number of parameters passed in PROGMAN DDE call (AddItem)
  char szParams[10][MAX_PATH]; // each arg is copied here after stripping
  char* pszParamList[10];      // This holds a list of pointers to args
  char* pszTmp;

  pszParamList[0] = pszParameters;

  int nCurIndex = 1;
  pszTmp = StrChr(pszParameters, ','); // find a delimiter
  while (NULL != pszTmp)
  {
    *(pszTmp++) = '\0'; // null terminate and cut.
    pszParamList[nCurIndex++] = pszTmp; // put it in the pointer list
    pszTmp = StrChr(pszTmp, ','); // get the next onf
  }

  for (int nLoop = 0; nLoop < nCurIndex; nLoop++)
  {
    lstrcpy(szParams[nLoop], pszParamList[nLoop]); // copy the parameter
    StrTrim(szParams[nLoop], "\t\r\n "); // Trim any nastys
    PathUnquoteSpaces(szParams[nLoop]); // remove any quotes
  }

  // Set initial state of the group to act on:
  // bCommon ? Perform function on a common group : Perform function on a private group
  BOOL bCommon = m_bIsUserAnAdmin;

  // DDE call: e.g. CreateGroup(GroupName[,CommonGroupFlag])
  if (0 == lstrcmp(szRequest, "CREATEGROUP"))
  {
    if (2 == nCurIndex) // second parameter forces common/private
    {
      bCommon = StrToInt(szParams[1]);
    }
    return (CreateGroup(szParams[0], bCommon));
  } else
  // DDE call: e.g. ExploreFolder(idlist[,object])
  if (0 == lstrcmp(szRequest, "EXPLOREFOLDER"))
  {
	  char szParam[MAX_PATH];
	  lstrcpy(szParam, "/e,/idlist,");
	  lstrcat(szParam, szParams[1]);
	  return ((ShellExecute(NULL, "open", "explorer.exe", szParam, NULL, SW_SHOW) > HINSTANCE(32)) ? TRUE : FALSE);
  } else 
  // DDE call: e.g. FindFolder(idlist[,object])
  if (0 == lstrcmp(szRequest, "FINDFOLDER"))
  {
	  char szParam[MAX_PATH];
	  lstrcpy(szParam, "/e,/idlist,");
	  lstrcat(szParam, szParams[1]);
	  return ((ShellExecute(NULL, "open", "explorer.exe", szParam, NULL, SW_SHOW) > HINSTANCE(32)) ? TRUE : FALSE);
  } else 
  // DDE call: e.g. ViewFolder(idlist[,object])
  if (0 == lstrcmp(szRequest, "VIEWFOLDER"))
  {
	  char szParam[MAX_PATH];
	  HINSTANCE ret;
	  lstrcpy(szParam, "/idlist,");
	  lstrcat(szParam, szParams[1]);
	  // Open Network and Dial-up Connections and Network Properties
	  // Will return too soon if we don't sleep here... funky
	  ret = ShellExecute(NULL, "open", "explorer.exe", szParam, NULL, SW_SHOW);
	  Sleep(100);
	  return ((ret > HINSTANCE(32)) ? TRUE : FALSE);
  } else     
  // DDE call: e.g. DeleteGroup(GroupName[,CommonGroupFlag])
  if (0 == lstrcmp(szRequest, "DELETEGROUP"))
  {
    if (2 == nCurIndex) // second parameter forces common/private
    {
      bCommon = StrToInt(szParams[1]);
    }
    return (DeleteGroup(szParams[0], bCommon));
  } else
  // DDE call: e.g. ShowGroup(GroupName,ShowCommand[,CommonGroupFlag])
  if (0 == lstrcmp(szRequest, "SHOWGROUP"))
  {
    if (nCurIndex < 2) // show command: maps to SW_
      return (FALSE);
    if (3 == nCurIndex) // third parameter forces common/private
    {
      bCommon = StrToInt(szParams[2]);
    }
    int nShow = StrToInt(szParams[1]);
    return (ShowGroup(szParams[0], nShow, bCommon));
  } else
  // DDE call: e.g. DeleteItem(ItemName) : applies to current group
  if (0 == lstrcmp(szRequest, "DELETEITEM"))
  {
    return (DeleteItem(szParams[0]));
  } else
  //DDE call: AddItem(CmdLine[,Name[,IconPath[,IconIndex[,xPos,yPos[,DefDir[,
  //HotKey[,fMinimize[fSeparateMemSpace] ] ] ] ] ] ]) : applies to current group
  if (0 == lstrcmp(szRequest, "ADDITEM"))
  {
    BOOL bSepMemSpace = FALSE;
    BOOL bMinimize = FALSE;
    WORD dwHotKey  = 0;
    char* pszDefDir = "";
    int nIconIndex = 0;
    char* pszIconPath = "";
    char* pszDescription = "";
    
    // fSeparateMemSpace ignored for now

    if (nCurIndex > 8)
    { // fMinimize
      bMinimize = (BOOL)StrToInt(szParams[8]);
    }
    if (nCurIndex > 7)
    { // HotKey
      dwHotKey = (WORD)StrToInt(szParams[7]);
    }
    if (nCurIndex > 6)
    { // DefDir
      pszDefDir = szParams[6];
    }
    if (nCurIndex > 3) //xPos and yPos ignored. Not necessary
    { // IconIndex
      nIconIndex = StrToInt(szParams[3]);
    }
    if (nCurIndex > 2)
    { // IconPath
      pszIconPath = szParams[2];
    }
    if (nCurIndex > 1)
    {
      pszDescription = szParams[1];
    }

    return (AddItem(szParams[0], pszDescription, pszIconPath, nIconIndex, pszDefDir, dwHotKey, bMinimize, bSepMemSpace));
  } 
  return (FALSE);  
}

// Function to get the path to the Programs directory: either common or private
BOOL CSGTShellFunc::GetProgramsPath(BOOL bCommon, char* szPath)
{
  LPITEMIDLIST   pidlStartMenu;

  // Get the special folder  IDLIST
  if (NOERROR != SHGetSpecialFolderLocation(NULL, bCommon ? CSIDL_COMMON_PROGRAMS : CSIDL_PROGRAMS, &pidlStartMenu))
  {
    return (FALSE);
  }

  // Convert IDLIST into a path
  if (FALSE == SHGetPathFromIDList(pidlStartMenu, szPath))
  {
    return (FALSE);
  }

  return (TRUE);
}

// Show the program group to the user
BOOL CSGTShellFunc::ShowGroup(LPCTSTR strGroupName, int nShow, BOOL bCommon)
{
  char           szTemp[MAX_PATH],
                 szPath[MAX_PATH];
  HANDLE          hFind;
  WIN32_FIND_DATA FindData;

  // Get the program group path
  if (!GetProgramsPath(bCommon, szPath))
  {
    return (FALSE);
  }

  // Append \*.* for FindFirstFile
  lstrcpy(szTemp, "\\");
  lstrcat(szTemp, strGroupName);
  lstrcat(szPath, szTemp);

  lstrcpy(szTemp, szPath);
  lstrcat(szTemp, "\\*.*");

  // Use FindFirstFile to determine if the directory exists
  hFind = FindFirstFile(szTemp, &FindData);

  BOOL bRet = (INVALID_HANDLE_VALUE != hFind);

  FindClose(hFind);   

  // open it up!
  ShellExecute(NULL, "open", szPath, NULL, NULL, nShow);

  if (TRUE == bRet)
  {
    // set our current group to this one, as per Progman DDE
    lstrcpy(m_szCurrentGroup, szPath);
  }

  return (bRet);
}

// Create the program group
BOOL CSGTShellFunc::CreateGroup(LPCTSTR strGroupName, BOOL bCommon)
{
  char           szTemp[MAX_PATH],
                 szPath[MAX_PATH];

  // Get the program group path
  if (!GetProgramsPath(bCommon, szPath))
  {
    return (FALSE);
  }

  lstrcpy(szTemp, "\\");
  lstrcat(szTemp, strGroupName);
  lstrcat(szPath, szTemp);

  // standard create directory call
  if (!CreateDirectory(szPath, NULL))
  {
    if (ERROR_ALREADY_EXISTS != GetLastError())
    return (FALSE);
  }

  lstrcpy(m_szCurrentGroup, szPath);
  // Hmmm should we show the group here???? Maybe
  // but not for now.
  //ShellExecute(NULL, "open", szPath, NULL, NULL, nShow);

  // Tell the shell that something changed
  SHChangeNotify(SHCNE_MKDIR, SHCNF_PATH, szPath, 0);
  return (TRUE);
}

// Remove the group
BOOL CSGTShellFunc::DeleteGroup(LPCTSTR strGroupName, BOOL bCommon)
{
  char            szTemp[MAX_PATH],
                  szPath[MAX_PATH];

  HANDLE          hFind;
  WIN32_FIND_DATA FindData;
  BOOL            bFindFile = TRUE;

  // Get the program group path
  if (!GetProgramsPath(bCommon, szPath))
  {
    return (FALSE);
  }

  // Append \*.* for FindFirstFile
  lstrcpy(szTemp, "\\");
  lstrcat(szTemp, strGroupName);
  lstrcat(szPath, szTemp);

  lstrcpy(szTemp, szPath);
  lstrcat(szTemp, "\\*.*");

  // Use FindFirstFile to list dir. contents
  hFind = FindFirstFile(szTemp, &FindData);

  // kill them all off
  while((INVALID_HANDLE_VALUE != hFind) && bFindFile)
  {
    if(*(FindData.cFileName) != '.')
    {
      lstrcpy(szTemp, szPath);
      lstrcat(szTemp, "\\");
      lstrcat(szTemp, FindData.cFileName);
      lstrcat(szTemp, "\0");

      DeleteFile(szTemp);
    }
    bFindFile = FindNextFile(hFind, &FindData);
  }

  FindClose(hFind);   

  return RemoveDirectory(szPath);

}

// helper function to do allocation for the list of program groups
BOOL CSGTShellFunc::ListGroupsHelper(HANDLE hHeap, char* szPath, LPVOID& pGroupList, UINT& ulSize)
{
  HANDLE          hFind;
  WIN32_FIND_DATA FindData;
  BOOL            bFindFile = TRUE;
  char*           pszTemp = NULL;

  // Get the first one
  hFind = FindFirstFile(szPath, &FindData);

  // allocate the group list with base zero terminator if NULL
  if (NULL == pGroupList) 
    pGroupList = HeapAlloc(hHeap, HEAP_ZERO_MEMORY, 1);

  // check allocation
  if (NULL == pGroupList)
  {
    return (FALSE);
  }

  // iterate through the files
  // copy their names into a string in which each field is terminated
  // by \r\n
  while((INVALID_HANDLE_VALUE != hFind) && bFindFile)
  {
    if (*(FindData.cFileName) != '.' && (FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0)
    {
      int nLen = lstrlen(FindData.cFileName);

      pszTemp = (char*) HeapReAlloc(hHeap, HEAP_ZERO_MEMORY, pGroupList, ulSize + nLen + 2);
      if (pszTemp != NULL)
      {
        CopyMemory(&pszTemp[ulSize], FindData.cFileName, nLen);
        CopyMemory(&pszTemp[ulSize + nLen], "\r\n", 2);
        pGroupList = (LPVOID) pszTemp;
        ulSize += (nLen + 2);
      }
      else
      {
        // Cleanup
        FindClose(hFind);
        return (FALSE);
      }
    }
    // continue to next
    bFindFile = FindNextFile(hFind, &FindData);
  }

  // Cleanup
  FindClose(hFind);
  return (TRUE);
}


// List the program groups
BOOL CSGTShellFunc::ListGroups(LPVOID& pGroupList, UINT& ulSize)
{
  char            szTemp[MAX_PATH],
                  szPath[MAX_PATH];


  pGroupList = NULL;
  ulSize = 0;
  char* pszTemp = NULL;

  // Get user specific folders
  if (!GetProgramsPath(FALSE, szPath))
  {
    return (FALSE);
  }

  lstrcpy(szTemp, szPath);
  lstrcat(szTemp, "\\*.*");

  HANDLE hHeap = GetProcessHeap();
  if (NULL == hHeap)
  {
    return (FALSE);
  }

  if (!ListGroupsHelper(hHeap, szTemp, pGroupList, ulSize))
  {
    return (FALSE);
  }

  // Get common folders
  if (!GetProgramsPath(TRUE, szPath))
  {
    return (FALSE);
  }

  lstrcpy(szTemp, szPath);
  lstrcat(szTemp, "\\*.*");

  if (!ListGroupsHelper(hHeap, szTemp, pGroupList, ulSize))
  {
    return (FALSE);
  }

  // add null terminator
  pszTemp = (char*) HeapReAlloc(hHeap, HEAP_ZERO_MEMORY, pGroupList, ulSize + 1);
  if (pszTemp != NULL)
  {
    pszTemp[ulSize] = '\0';
    pGroupList = (LPVOID) pszTemp;
    ulSize += 1;
  }
  else
  {
    return (FALSE);
  }

  return (TRUE);
}


// kill an item
BOOL CSGTShellFunc::DeleteItem(LPCTSTR strItem)
{
  char            szPath[MAX_PATH];

  if (0 == lstrlen(m_szCurrentGroup))
    return (FALSE);

  lstrcpy(szPath, m_szCurrentGroup);
  lstrcat(szPath, "\\");
  lstrcat(szPath, strItem);

  return (DeleteFile(szPath));
}

// Add an item
BOOL CSGTShellFunc::AddItem(LPCTSTR strCmdLine, LPCTSTR strDescription, LPCTSTR strIconPath, int nIconIndex, LPCTSTR strDefDir, WORD dwHotKey, BOOL bMinimize, BOOL bSepMemSpace)
{
  char            szPath[MAX_PATH];
  HRESULT         hr; 
  IShellLink*     pShellLink;
  IPersistFile*   pPersistFile;
  

  // check that we've set a current group via showgroup or creategroup
  if (0 == lstrlen(m_szCurrentGroup))
    return (FALSE);

  
  char szDesc[MAX_PATH];
  char szArgs[MAX_PATH];
  char szCmd[MAX_PATH];

  PathQuoteSpaces((char*) strCmdLine);

  LPTSTR strArgs = PathGetArgs(strCmdLine);
  if (NULL != strArgs)
  {
    lstrcpy(szArgs, strArgs);
    PathRemoveArgs((char*)strCmdLine);
  }

  PathUnquoteSpaces((char*) strCmdLine);
  lstrcpy(szCmd, strCmdLine);

  if (0 == lstrlen(strDescription))
  {
    lstrcpy(szDesc, PathFindFileName(strCmdLine));
    PathRemoveExtension(szDesc);
  }
  else
  {
    PathUnquoteSpaces((char*) strDescription);
    lstrcpy(szDesc, strDescription);
  }

  lstrcpy(szPath, m_szCurrentGroup);
  lstrcat(szPath, "\\");
  lstrcat(szPath, szDesc);
  lstrcat(szPath, ".lnk");

  hr = CoCreateInstance(CLSID_ShellLink, 
                        NULL, 
                        CLSCTX_INPROC_SERVER, 
                        IID_IShellLink, 
                        (LPVOID*)&pShellLink); 

  if (S_OK != hr)
  {
    return (FALSE);
  }

  // Set the shortcut info.
  pShellLink->SetDescription(szDesc);
  pShellLink->SetHotkey(dwHotKey);
  pShellLink->SetWorkingDirectory(strDefDir);
  pShellLink->SetShowCmd(bMinimize ? SW_SHOWMINIMIZED : SW_SHOWNORMAL);
  pShellLink->SetArguments(szArgs);


  pShellLink->SetPath(szCmd);

  if (0 != lstrlen(strIconPath))
  {
    pShellLink->SetIconLocation(strIconPath, nIconIndex);
  }

  // Save it.
  hr = pShellLink->QueryInterface(IID_IPersistFile, (LPVOID*)&pPersistFile); 

  if (S_OK == hr) 
  { 
    WCHAR wsz[MAX_PATH]; 

    // Ensure that the string is ANSI. 
    MultiByteToWideChar(CP_ACP, 0, szPath, -1, wsz, MAX_PATH); 

    // Save the link by calling IPersistFile::Save. 
    hr = pPersistFile->Save(wsz, TRUE); 

    if (S_OK != hr)
    {
      pShellLink->Release();
      pPersistFile->Release(); 
      return (FALSE);
    }
    pPersistFile->Release(); 
  } 

  pShellLink->Release();
  SHChangeNotify(SHCNE_UPDATEDIR, SHCNF_PATH, m_szCurrentGroup, 0);
  SHChangeNotify(SHCNE_CREATE, SHCNF_PATH, szPath, 0);
  return (TRUE);
}

